///////////////////////////////////////////////////////////////////////////////////
//No part of this file can be copied or released without the consent of 
//Avalanche Technology
//										
///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////
//										
//	Avalanche Technology Inc., Proprietary and Confidential	   *
//										
// 	Release:  2.1    Date 7/21/2021 	
//										
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
//  PART DESCRIPTION:
//
//  Technology: 40nm pMTJ STT-MRAM
//  Part:       AS3016A04
//
//  Description: 16 Megabit HIGH PERFORMANCE SERIAL PERSISTANT SRAM MEMORY
//
////////////////////////////////////////////////////////////////////////////////////
//  FILE CONTENTS : STIMULUS MODULE
//
////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////
// MODULE DECLARATION                                                             //
////////////////////////////////////////////////////////////////////////////////////

`timescale 100ps / 10ps

module stimulus 
	(
		SO_IO1,
		CLK,
		IO3, 
		SI_IO0 ,
		WPn_IO2,
		CSn
	);

        inout SO_IO1;
	inout IO3;
	inout SI_IO0;
	inout WPn_IO2;
	
	output CSn;
	output CLK;
        
	reg CSn = 1;
	reg CLK = 0;
        
	reg SI_IO0_reg=0;
        reg SO_IO1_reg=0;
        reg WPn_IO2_reg=0;
        reg IO3_reg=0;
        
	integer tx=0;
	reg [2:0] Mode;

        parameter SPI = 3'h01;
        parameter DPI = 3'h02;
        parameter QPI = 3'h04;

	`include "./task.v"
	 
	 assign SI_IO0 = tx ? SI_IO0_reg : 1'bz;  
	 assign SO_IO1 = tx ? SO_IO1_reg : 1'bz;  
         assign WPn_IO2 = tx? WPn_IO2_reg : 1'bz;  
         assign IO3 = tx ? IO3_reg : 1'bz;  

	 initial 
  	 begin	
		#10000;
       	        $display("Current mode is SPI");  
		Status_Register_Check(SPI);
		Config_Register_Check(SPI);
		Memory_Write_Read_Check(SPI);

		DPI_Enable(SPI);
       	        $display("Current mode is DPI");  
		Status_Register_Check(DPI);
		Config_Register_Check(DPI);
		Memory_Write_Read_Check(DPI);

		QPI_Enable(DPI);
       	        $display("Current mode is QPI");  
		Status_Register_Check(QPI);
		Config_Register_Check(QPI);
		Memory_Write_Read_Check(QPI);


		#10000;
		$finish();
  	 end

endmodule 
