///////////////////////////////////////////////////////////////////////////////////
//No part of this file can be copied or released without the consent of 
//Avalanche Technology
//										
///////////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////////
//										
//	Avalanche Technology Inc., Proprietary and Confidential	   *
//										
// 	Release:  2.1    Date 7/21/2021  
//										
////////////////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////////
//  PART DESCRIPTION:
//
//  Technology: 40nm pMTJ STT-MRAM
//  Part:       AS3016A04
//
//  Description: 16 Megabit HIGH PERFORMANCE SERIAL PERSISTANT SRAM MEMORY
//
////////////////////////////////////////////////////////////////////////////////////
//  FILE CONTENTS : SPI/DPI/QPI RELATED TASKS 
//
////////////////////////////////////////////////////////////////////////////////////


////////////////////////////////////////////////////////////////////////////////////
// MODULE DECLARATION                                                             //
////////////////////////////////////////////////////////////////////////////////////

`timescale 100ps / 10ps

/////////////////////////////////////////////////////////////////
//   Datasheet Timing
/////////////////////////////////////////////////////////////////
specify 
	specparam tPOR_time  = 1; 
	specparam tCSS       = 50; 
	specparam tCSH       = 40; 
	specparam tCH        = 92; 
	specparam tCL        = 92; 
	specparam tCS        = 1300; 
	specparam tSU        = 20;
	specparam tHD        = 20;
	specparam tCO        = 70;
endspecify

/////////////////////////////////////////////////////////////////
// Select chip
/////////////////////////////////////////////////////////////////
task SelectChip;
begin
	CSn = 0;
	#(tCSS);
end
endtask 
/////////////////////////////////////////////////////////////////
// Deselect chip
/////////////////////////////////////////////////////////////////
task DeSelChip;
begin
	#(tCSH);
    	CSn = 1;
	#(tCS);
end
endtask
/////////////////////////////////////////////////////////////////
// Send byte
////////////////////////////////////////////////////////////////
task SendByte;
input [7:0] data;
integer i;
begin
	CLK = 0;
        tx=1;
	for(i=0; i<8; i=i+1)
	begin
  		SI_IO0_reg = data[7-i];
		#(tSU);
		CLK = 1;
		#(tHD)
   		SI_IO0_reg = 0;
		#(tCH-tHD);
		CLK = 0;
		#(tCL-tSU);
	end
     	tx=0;
end
endtask 
/////////////////////////////////////////////////////////////////
// Send Dual byte
/////////////////////////////////////////////////////////////////
task SendByteDual;
input [7:0] data;
integer i;
begin
	CLK = 0;
        tx=1;
	for(i=0; i<8; i=i+2)
	begin
  		SO_IO1_reg = data[7-i];
                SI_IO0_reg = data[7-1-i];
		#(tSU);
		CLK = 1;
		#(tHD)
   		SO_IO1_reg = 0;
                SI_IO0_reg = 0;
		#(tCH-tHD);
		CLK = 0;
		#(tCL-tSU);
	end
 	tx=0;
end
endtask 
/////////////////////////////////////////////////////////////////
// Send Quad byte
/////////////////////////////////////////////////////////////////
task SendByteQuad;
input [7:0] data;
integer i;
begin
	CLK = 0;
        tx=1;
	for(i=0; i<8; i=i+4)
	begin
  		IO3_reg = data[7-i];
                WPn_IO2_reg = data[7-1-i];
                SO_IO1_reg = data[7-2-i];
                SI_IO0_reg = data[7-3-i];
		#(tSU);
		CLK = 1;
		#(tHD)
                IO3_reg = 0; 
                WPn_IO2_reg = 0;
                SO_IO1_reg = 0;
   		SI_IO0_reg = 0;
		#(tCH-tHD);
		CLK = 0;
		#(tCL-tSU);
	end
	tx=0;
end
endtask 
/////////////////////////////////////////////////////////////////
// Task to receive byte
/////////////////////////////////////////////////////////////////
task RcvByte;
output [7:0] data;
integer i;
begin
	CLK = 0;
        tx=0;
	for(i=0; i<8; i=i+1)
	begin
  		SI_IO0_reg = 0;
		#(tSU);
		CLK = 1;
		#(tHD)
   		SI_IO0_reg = 0;
		#(tCH-tHD);
                SO_IO1_reg = 1'bz;
         	SI_IO0_reg = 0;
		data[7-i] = SO_IO1;
		CLK = 0;
		#(tCL-tSU);
	end
end
endtask 
/////////////////////////////////////////////////////////////////
// Task to receive Dualbyte
/////////////////////////////////////////////////////////////////
task RcvByteDual;
output [7:0] data;
integer i;
begin
	CLK = 0;
        tx=0;
	for(i=0; i<8; i=i+2)
	begin
	        SO_IO1_reg = 0; 
  		SI_IO0_reg = 0;  	
		#(tSU);
		CLK = 1;
		#(tHD)
          	SO_IO1_reg = 0;   		
                SI_IO0_reg = 0;   	
		#(tCH-tHD);
                SO_IO1_reg=1'bz;
                SI_IO0_reg=1'bz;		
		data[7-i] = SO_IO1;
		data[7-1-i] = SI_IO0;
		CLK = 0;
		#(tCL-tSU);
	end
end
endtask 
/////////////////////////////////////////////////////////////////
// Task to receive Quadbyte
/////////////////////////////////////////////////////////////////
task RcvByteQuad;
output [7:0] data;
integer i;
begin
	CLK = 0;
        tx=0;
	for(i=0; i<8; i=i+4)
	begin
  	        IO3_reg = 0; 
                WPn_IO2_reg = 0;
  		SO_IO1_reg = 0; 
  		SI_IO0_reg = 0; 
		#(tSU);
		CLK = 1;
		#(tHD)
   	        IO3_reg = 0; 
                WPn_IO2_reg = 0;
                SO_IO1_reg = 0;
   		SI_IO0_reg = 0;
		#(tCH-tHD);
                IO3_reg=1'bz;
                WPn_IO2_reg=1'bz;
                SO_IO1_reg=1'bz;
                SI_IO0_reg=1'bz;
                data[7-i]= IO3;
                data[7-1-i]= WPn_IO2;
                data[7-2-i]= SO_IO1;
                data[7-3-i]= SI_IO0;
		CLK = 0;
		#(tCL-tSU);
	end
end
endtask 
/////////////////////////////////////////////////////////////////
// Send command
/////////////////////////////////////////////////////////////////
task Cmd;
input [2:0] Mode;
input [7:0] data;
begin
	if (Mode == 3'h01)
	begin
	SendByte(data);
	end
     
   	else if (Mode == 3'h02)
	begin
	SendByteDual(data);
	end
          
        else if (Mode == 3'h04)
	begin
	SendByteQuad(data);
	end
end
endtask
/////////////////////////////////////////////////////////////////
// Send 24-bit address
/////////////////////////////////////////////////////////////////
task Addr_24bit;
input [2:0] Mode;
input [23:0] addr;
begin
	if (Mode == 3'h01)
	begin
	SendByte(addr[23:16]);
	SendByte(addr[15:8]);
	SendByte(addr[7:0]);
	end

	else if (Mode == 3'h02)
	begin
	SendByteDual(addr[23:16]);
	SendByteDual(addr[15:8]);
	SendByteDual(addr[7:0]);
	end

     	else if (Mode == 3'h04)
	begin
	SendByteQuad(addr[23:16]);
	SendByteQuad(addr[15:8]);
	SendByteQuad(addr[7:0]);
	end
end
endtask
/////////////////////////////////////////////////////////////////
// Task to send mode byte
/////////////////////////////////////////////////////////////////
task ModeByte;
input [2:0] Mode;
input [7:0] data;
begin
	if (Mode == 3'h01)
	begin
	SendByte(data);
	end

	else if (Mode == 3'h02)
	begin
	SendByteDual(data);
	end
      
        else if (Mode == 3'h04)
	begin
	SendByteQuad(data);
	end
end
endtask
/////////////////////////////////////////////////////////////////
// Task to send latency
/////////////////////////////////////////////////////////////////
task Latency;
input [7:0] num;
integer i;
begin
	CLK = 0;
	for(i=0; i<num; i=i+1)
	begin
  		IO3_reg =1'bz;
		WPn_IO2_reg =1'bz;
		SO_IO1_reg = 1'bz;
		SI_IO0_reg = 1'bz;
		#(tSU);
		CLK = 1;
		#(tHD)
   		IO3_reg =1'bz;
		WPn_IO2_reg =1'bz;
		SO_IO1_reg = 1'bz;
		SI_IO0_reg = 1'bz;
		#(tCH-tHD);
		CLK = 0;
		#(tCL-tSU);
	end
end
endtask
/////////////////////////////////////////////////////////////////
// SPI enable for Modes
/////////////////////////////////////////////////////////////////
task SPIEnable;
input [2:0] Mode;
begin
    SelectChip();
    Cmd(Mode,8'hff);
    DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// DPI enable for Modes
/////////////////////////////////////////////////////////////////
task DPIEnable;
input [2:0] Mode;
begin
    SelectChip();
    Cmd(Mode,8'h37);
    DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// QPI enable for Modes
/////////////////////////////////////////////////////////////////
task QPIEnable;
input [2:0] Mode;
begin
    SelectChip();
    Cmd(Mode,8'h38);
    DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// Write enable
/////////////////////////////////////////////////////////////////
task WriteEnable;
input [2:0] Mode;
begin
    	SelectChip();
    	Cmd(Mode,8'h06);
    	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// SPI enable
/////////////////////////////////////////////////////////////////
task SPI_Enable;
input [2:0] Mode;
begin
    	SelectChip();
    	Cmd(Mode,8'hff);
    	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// DPI enable
/////////////////////////////////////////////////////////////////
task DPI_Enable;
input [2:0] Mode;
begin
    	SelectChip();
    	Cmd(Mode,8'h37);
    	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// QPI enable
/////////////////////////////////////////////////////////////////
task QPI_Enable;
input [2:0] Mode;
begin
    	SelectChip();
    	Cmd(Mode,8'h38);
    	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// Write disable
/////////////////////////////////////////////////////////////////
task WriteDisable;
input [2:0] Mode;
begin
	SelectChip();
    	Cmd(Mode,8'h04);
    	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
//  Write status
/////////////////////////////////////////////////////////////////
task WriteStatus();
input [2:0] Mode;
input [7:0]data;
begin
    	SelectChip();
    	Cmd(Mode,8'h01);
	
	if (Mode == 3'h01)
	begin
    	SendByte(data);
	end

    	else if (Mode == 3'h02)
	begin
    	SendByteDual(data);
	end
        
     	else if (Mode == 3'h04)
	begin
    	SendByteQuad(data);
	end
    	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// Write Config
/////////////////////////////////////////////////////////////////
task WriteConfig();
input [2:0] Mode;
input [7:0] data1;
input [7:0] data2;
input [7:0] data3;
input [7:0] data4;
begin
   	SelectChip();
   	Cmd(Mode,8'h87);
	
	if (Mode == 3'h01)
	begin
   	SendByte(data1);
   	SendByte(data2);
   	SendByte(data3);
   	SendByte(data4);
	end

	else if (Mode == 3'h02)
	begin
   	SendByteDual(data1);
   	SendByteDual(data2);
   	SendByteDual(data3);
   	SendByteDual(data4);
	end
        
      	else if (Mode == 3'h04)
	begin
   	SendByteQuad(data1);
   	SendByteQuad(data2);
   	SendByteQuad(data3);
   	SendByteQuad(data4);
	end
   	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// Read status
/////////////////////////////////////////////////////////////////
task ReadStatus();
input [2:0] Mode;
input [7:0] data;
reg [7:0] data_rcv;
begin
    	SelectChip();
    	Cmd(Mode,8'h05);
	
	if (Mode == 3'h01)
	begin
    	RcvByte(data_rcv);
	end

        else if (Mode == 3'h02)
	begin
    	RcvByteDual(data_rcv);
	end
       
	else if (Mode == 3'h04)
	begin
    	RcvByteQuad(data_rcv);
	end

    	DeSelChip();
	
	if(data_rcv !== data & 8'hfc)
		$display("[ERROR] Status Compare Error : Expected %x Received %x",data,data_rcv);
	else
		$display("[INFO] Status Compare Pass : Expected %x Received %x",data,data_rcv);
end
endtask
/////////////////////////////////////////////////////////////////
// Read Config
/////////////////////////////////////////////////////////////////
task ReadConfig ();
input [2:0] Mode;
input [7:0] data1;
input [7:0] data2;
input [7:0] data3;
input [7:0] data4;
reg [7:0] data_rcv1;
reg [7:0] data_rcv2;
reg [7:0] data_rcv3;
reg [7:0] data_rcv4;
begin
   	SelectChip();
   	Cmd(Mode,8'h46);
       
 	if (Mode == 3'h01)
	begin
   	RcvByte(data_rcv1);
   	RcvByte(data_rcv2);
   	RcvByte(data_rcv3);
   	RcvByte(data_rcv4);
	end

        else if (Mode == 3'h02)
	begin
   	RcvByteDual(data_rcv1);
   	RcvByteDual(data_rcv2);
   	RcvByteDual(data_rcv3);
   	RcvByteDual(data_rcv4);
	end
     
        else if (Mode == 3'h04)
	begin
   	RcvByteQuad(data_rcv1);
   	RcvByteQuad(data_rcv3);
   	RcvByteQuad(data_rcv2);
   	RcvByteQuad(data_rcv4);
	end
     	DeSelChip();

	
		if(data_rcv1 !== data1 || (data_rcv2 & 8'haf) !== (data2 & 8'haf) || data_rcv3 !== data3 || data_rcv4 !== data4)
              
			$display("[ERROR] CFG Reg Compare Error : Expected %x %x %x %x Received %x %x %x %x",data1, data2, data3, data4, data_rcv1, data_rcv2, data_rcv3, data_rcv4);
		else
			$display("[INFO] CFG Reg Compare Pass : Expected %x %x %x %x Received %x %x %x %x",data1, data2, data3, data4, data_rcv1, data_rcv2, data_rcv3, data_rcv4);

   
end
endtask
/////////////////////////////////////////////////////////////////
// Memory Write
/////////////////////////////////////////////////////////////////
task Write();
input [2:0] Mode;
input [23:0] addr;
input [7:0] data;
input [23:0] num_data_i;
integer i; 
begin
	$display("[INFO] Mem write  : Address %x data %x num_data %x",addr, data, num_data_i);
    	SelectChip();
    	Cmd(Mode,8'h02);
    	Addr_24bit(Mode,addr);
	for (i=0;i<num_data_i;i=i+1)
	begin
	
	if (Mode == 3'h01)
	begin
	SendByte(data);
	end

        else if (Mode == 3'h02)
	begin
	SendByteDual(data);
	end
        
        else if (Mode == 3'h04)
	begin
	SendByteQuad(data);
	end

 	end
    	DeSelChip();
end
endtask

/////////////////////////////////////////////////////////////////
// Memory Read
/////////////////////////////////////////////////////////////////
task Read();
input [2:0] Mode;
input [23:0] addr;
input [7:0] data;
input [7:0] latency_val;
input [23:0] num_data_o;
integer i;
reg [7:0] data_rcv;
begin
	$display("[INFO] Mem read  : Address %x data %x latency %x num_data %x",addr, data, latency_val, num_data_o);
    	SelectChip();
    	Cmd(Mode,8'h03);
    	Addr_24bit(Mode,addr);
    	Latency(latency_val);
    	for (i=0;i<num_data_o;i=i+1)
	begin
           
	if (Mode == 3'h01)
	begin
	RcvByte(data_rcv);
	end

        else if (Mode == 3'h02)
	begin
	RcvByteDual(data_rcv);
	end
       
	else if (Mode == 3'h04)
	begin
	RcvByteQuad(data_rcv);
	end

		if(data_rcv !== data)
			$display("[ERROR] Mem Read Compare Error : Expected %x Received %x",data,data_rcv);
		else
			$display("[INFO] Mem Read Compare Pass : Expected %x Received %x",data,data_rcv);
	end
    	DeSelChip();
end
endtask

/////////////////////////////////////////////////////////////////
// Memory Read using Fast Read
/////////////////////////////////////////////////////////////////
task FastRead();
input [2:0] Mode;
input [23:0] addr;
input [7:0] data;
input [7:0] latency_val;
input [23:0] num_data_o;
integer i;
reg [7:0] data_rcv;
begin
	$display("[INFO] Mem read  : Address %x data %x latency %x num_data %x",addr, data, latency_val, num_data_o);
    	SelectChip();
    	Cmd(Mode,8'h0B);
    	Addr_24bit(Mode,addr);
	ModeByte(Mode,8'hFF);
    	Latency(latency_val);
    	for (i=0;i<num_data_o;i=i+1)
	begin
        
	if (Mode == 3'h01)
	begin
	RcvByte(data_rcv);
	end

        else if (Mode == 3'h02)
	begin
	RcvByteDual(data_rcv);
	end
       
	else if (Mode == 3'h04)
	begin
	RcvByteQuad(data_rcv);
	end
	
		if(data_rcv !== data)
			$display("[ERROR] Mem Fast Read Compare Error : Expected %x Received %x",data,data_rcv);
		else
			$display("[INFO] Mem Fast Read Compare Pass : Expected %x Received %x",data,data_rcv);
	end
    	DeSelChip();
end
endtask
/////////////////////////////////////////////////////////////////
// Status register write read check
/////////////////////////////////////////////////////////////////
task Status_Register_Check();
input [2:0] Mode;
begin 
   	parameter SPI = 3'h01;
        parameter DPI = 3'h02;
        parameter QPI = 3'h04;
       
        if(Mode == 3'h01)
	begin
	WriteEnable(SPI);
   	WriteStatus(SPI,8'hFF);
   	ReadStatus(SPI,8'hFF);
   	
	WriteStatus(SPI,8'haa);
   	ReadStatus(SPI,8'hFF);
	end

        else if(Mode  == 3'h02)
	begin
	WriteEnable(DPI);
   	WriteStatus(DPI,8'hFF);
   	ReadStatus(DPI,8'hFF);
   	
	WriteStatus(DPI,8'haa);
   	ReadStatus(DPI,8'hFF);
        end
	 
        else if(Mode == 3'h04)
	begin
	WriteEnable(QPI);
   	WriteStatus(QPI,8'hFF);
   	ReadStatus(QPI,8'hFF);
   	
	WriteStatus(QPI,8'haa);
   	ReadStatus(QPI,8'hFF);
        end
end
endtask
/////////////////////////////////////////////////////////////////
// Config register write read check
/////////////////////////////////////////////////////////////////
task Config_Register_Check();
input [2:0] Mode;
begin 
	parameter SPI = 3'h01;
        parameter DPI = 3'h02;
        parameter QPI = 3'h04;

	$display("*****************************************************************");
	$display("** Config register write read check");
	$display("*****************************************************************");
        if(Mode == 3'h01)
	begin
   	WriteEnable(SPI);
   	WriteConfig(SPI,8'hFF,8'hFF,8'hFF,8'hFF);
   	ReadConfig(SPI,8'hFF,8'hFF,8'hFF,8'hFF);

   	WriteConfig(SPI,8'haa,8'haa,8'haa,8'haa);
   	ReadConfig(SPI,8'hFF,8'hFF,8'hFF,8'hFF);

  	WriteEnable(SPI);
  	WriteDisable(SPI);
   	WriteConfig(SPI,8'haa,8'haa,8'haa,8'haa);
   	ReadConfig(SPI,8'hFF,8'hFF,8'hFF,8'hFF);

  	WriteEnable(SPI);
   	WriteConfig(SPI,8'h00,8'h08,8'h00,8'h00);
   	ReadConfig(SPI,8'h00,8'h08,8'h00,8'h00);

        end 

        else if(Mode == 3'h02)
	begin
	WriteEnable(DPI);
   	WriteConfig(DPI,8'hFF,8'hFF,8'hFF,8'hFF);
   	ReadConfig(DPI,8'hFF,8'hFF,8'hFF,8'hFF);

   	WriteConfig(DPI,8'haa,8'haa,8'haa,8'haa);
   	ReadConfig(DPI,8'hFF,8'hFF,8'hFF,8'hFF);

  	WriteEnable(DPI);
  	WriteDisable(DPI);
   	WriteConfig(DPI,8'haa,8'haa,8'haa,8'haa);
   	ReadConfig(DPI,8'hFF,8'hFF,8'hFF,8'hFF);

  	WriteEnable(DPI);
   	WriteConfig(DPI,8'h00,8'h08,8'h00,8'h00);
   	ReadConfig(DPI,8'h00,8'h08,8'h00,8'h00);
        end

        else if(Mode == 3'h04)
	begin   	
	WriteEnable(QPI);
   	WriteConfig(QPI,8'hFF,8'hFF,8'hFF,8'hFF);
   	ReadConfig(QPI,8'hFF,8'hFF,8'hFF,8'hFF);

   	WriteConfig(QPI,8'haa,8'haa,8'haa,8'haa);
   	ReadConfig(QPI,8'hFF,8'hFF,8'hFF,8'hFF);

  	WriteEnable(QPI);
  	WriteDisable(QPI);
   	WriteConfig(QPI,8'haa,8'haa,8'haa,8'haa);
   	ReadConfig(QPI,8'hFF,8'hFF,8'hFF,8'hFF);

  	WriteEnable(QPI);
   	WriteConfig(QPI,8'h00,8'h08,8'h00,8'h00);
   	ReadConfig(QPI,8'h00,8'h08,8'h00,8'h00);
        end
end
endtask
/////////////////////////////////////////////////////////////////
// Write Read Memory check
/////////////////////////////////////////////////////////////////
task Memory_Write_Read_Check();
input [2:0] Mode;
begin 

        parameter SPI = 3'h01;
        parameter DPI = 3'h02;
        parameter QPI = 3'h04;
        
	$display("*****************************************************************");
	$display("** Write Read Memory check ");
	$display("*****************************************************************");
        if(Mode == SPI)
	begin
   	WriteEnable(SPI);
   	WriteStatus(SPI,8'h00);
   	ReadStatus(SPI,8'h00);

   	WriteEnable(SPI);
   	WriteConfig(SPI,8'h00,8'h08,8'h00,8'h00);
   	ReadConfig(SPI,8'h00,8'h08,8'h00,8'h00);

   	WriteEnable(SPI);
   	Write(SPI,24'h000000,8'ha3,8'h0f);
   	Read(SPI,24'h000000,8'ha3,8'h08,8'h0f);
  	FastRead(SPI,24'h000000,8'ha3,8'h08,8'h0f);
        end
   	
        else if(Mode == DPI)
	begin
	WriteEnable(DPI);
   	WriteStatus(DPI,8'h00);
   	ReadStatus(DPI,8'h00);

   	WriteEnable(DPI);
   	WriteConfig(DPI,8'h00,8'h08,8'h00,8'h00);
   	ReadConfig(DPI,8'h00,8'h08,8'h00,8'h00);

   	WriteEnable(DPI);
   	Write(DPI,24'h000100,8'hb4,8'h0f);
   	//Read(DPI,24'h000100,8'hb4,8'h08,8'h0f);
  	FastRead(DPI,24'h000100,8'hb4,8'h08,8'h0f);

   	//Read(DPI,24'h000000,8'ha3,8'h08,8'h0f);
  	FastRead(DPI,24'h000000,8'ha3,8'h08,8'h0f);

        end
   	
        else if(Mode == QPI)
	begin
        WriteEnable(QPI);
   	WriteStatus(QPI,8'h00);
   	ReadStatus(QPI,8'h00);

   	WriteEnable(QPI);
   	WriteConfig(QPI,8'h00,8'h08,8'h00,8'h00);
   	ReadConfig(QPI,8'h00,8'h08,8'h00,8'h00);

   	WriteEnable(QPI);
   	Write(QPI,24'h000200,8'hc5,8'h0f);

   	//Read(QPI,24'h000200,8'hc5,8'h08,8'h0f);
  	FastRead(QPI,24'h000200,8'hc5,8'h08,8'h0f);

   	//Read(QPI,24'h000100,8'hb4,8'h08,8'h0f);
  	FastRead(QPI,24'h000100,8'hb4,8'h08,8'h0f);

   	//Read(QPI,24'h000000,8'ha3,8'h08,8'h0f);
  	FastRead(QPI,24'h000000,8'ha3,8'h08,8'h0f);

   	WriteEnable(QPI);
   	WriteConfig(QPI,8'h00,8'h0c,8'h00,8'h00);
   	ReadConfig(QPI,8'h00,8'h0c,8'h00,8'h00);


   	//Read(QPI,24'h000200,8'hc5,8'h0c,8'h0f);
  	FastRead(QPI,24'h000200,8'hc5,8'h0c,8'h0f);

   	//Read(QPI,24'h000100,8'hb4,8'h0c,8'h0f);
  	FastRead(QPI,24'h000100,8'hb4,8'h0c,8'h0f);

   	//Read(QPI,24'h000000,8'ha3,8'h0c,8'h0f);
  	FastRead(QPI,24'h000000,8'ha3,8'h0c,8'h0f);

   	WriteEnable(QPI);
   	WriteConfig(QPI,8'h00,8'h08,8'h00,8'h00);
   	ReadConfig(QPI,8'h00,8'h08,8'h00,8'h00);

   	WriteEnable(QPI);
   	Write(QPI,24'h100002,8'hc5,8'h0f);
   	//Read(QPI,24'h100002,8'hc5,8'h08,8'h0f);

   	WriteEnable(QPI);
   	WriteConfig(QPI,8'h00,8'h00,8'h00,8'h00);
   	ReadConfig(QPI,8'h00,8'h00,8'h00,8'h00);

	SPI_Enable(QPI);
   	Read(SPI,24'h100002,8'hc5,8'h00,8'h0f);

        end
end
endtask


